﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.IO.Ports;
using System.Windows.Input;
using System.IO;
using System.Net;
using System.Net.Sockets;

namespace SNESControllerServer
{
    class Program
    {    
        static void Main(string[] args)
        {
            try
            {
                string serverIP = "";
                string comPort = "";

                //Deal with serialport stuff
                string[] lines = System.IO.File.ReadAllLines(@"com.txt");

                foreach (string line in lines)
                {
                    comPort = line;
                }

                SerialPort serialPort = new SerialPort();
                serialPort.BaudRate = 9600;
                serialPort.PortName = comPort;
                serialPort.Open();
                serialPort.Write("");
                
                //Get the server IP address
                string[] lines2 = System.IO.File.ReadAllLines(@"serverip.txt");

                foreach (string line in lines2)
                {
                    serverIP = line;
                }

                //Some introductory text
                Console.WriteLine("SNES CONTROLLER EMULATOR SERVER");
                Console.WriteLine("Version 1.1");
                Console.WriteLine("by Riley Johanson - r.lagserv.net");
                Console.WriteLine("The arduino is connected to: " + comPort + " (change in com.txt)");
                Console.WriteLine("Server IP Address: " + serverIP + " (change in serverip.txt)");
                Console.WriteLine("The server is running at port 80.\n");

                //The main loop that responds to input from the client
                for (; ; )
                {
                    //Set up the IP Address and whatnot
                    IPAddress ipAd = IPAddress.Parse(serverIP);
                    TcpListener myList = new TcpListener(ipAd, 80);

                    // Start listening at the specified port 
                    myList.Start();
                    Console.WriteLine("Waiting for a connection.....");

                    //Accept socket
                    Socket s = myList.AcceptSocket();
                    Console.WriteLine("Connection accepted from " + s.RemoteEndPoint);

                    //Process input and send it to Arduino 
                    byte[] b = new byte[100];
                    int k = s.Receive(b);

                    string buttonInput = Encoding.UTF8.GetString(b, 0, b.Length);

                    Console.WriteLine("INPUT: " + buttonInput);

                    serialPort.Write(buttonInput);
                    Thread.Sleep(1);

                    //Clean up
                    s.Close();
                    myList.Stop();
                }
            }
            catch (Exception e)
            {
                //Report that there is an error and suggest some possible reasons why
                Console.WriteLine("ERROR.\n");
                Console.WriteLine("Make sure the server IP address in 'serverip.txt' is correct.");
                Console.WriteLine("Make sure the COM port in 'com.txt' is correct.");
                Console.WriteLine("Make sure the arduino is connected and powered on.");
                Console.WriteLine("Make sure 'serverip.txt' and 'com.txt' are both located in the same directory\nas this .exe.\n");
                Console.WriteLine("PRESS ENTER TO QUIT.");          
            }
            Console.ReadLine();
        }
    }
}
